'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (configName) {
  var configPath = getConfigPath(configName);
  var config = configPath ? require(configPath) : {};
  return { config: config, configPath: configPath };
};

var _ = require('lodash');

var _require = require('path');

var join = _require.join;

var cwd = require('cwd');
var fs = require('fs');
var debug = require('debug')('root-config');

// Try to get the config from some preset locations.
function getConfigPath(configName) {

  if (!configName) throw new Error('You must provide a config name to root-config.');

  var envName = process.env[_.upperCase(configName) + '_CONFIG_PATH']; // FOO_CONFIG_PATH
  debug('Trying:', 'process.env');
  if (envName) return envName;

  {
    var file = join(cwd(), '.' + configName + '.js'); // .foo.js
    debug('Trying:', file);
    if (fs.existsSync(file)) return file;
  }

  {
    var _file = join(cwd(), configName + '.js'); // foo.js
    debug('Trying:', _file);
    if (fs.existsSync(_file)) return _file;
  }

  {
    var _file2 = join(cwd(), 'tools', '.' + configName + '.js'); // tools/foo.js
    debug('Trying:', _file2);
    if (fs.existsSync(_file2)) return _file2;
  }

  // TODO(vjpr): __filename should be a ide link. There is a module I made for that.
  console.warn('No ' + configName + ' config file found. See ' + __filename + '.');

  return null;
}

module.exports = exports['default'];