'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.task = task;
var path = require('path');
var _ = require('lodash');
var cwd = require('cwd');

function getOpts(opts) {

  var srcDir = cwd('modules');
  var destDir = 'lib-modules';

  // TODO(vjpr): This is very bad! We should make each task generic. Should export just functions.
  // Or this might be fine.
  opts = _.merge({}, opts, {
    srcGlob: srcDir + '/**/*.js',
    srcFilesGlob: srcDir + '/**/!(*.js|*.jsx)',
    destGlob: destDir + '/**/*',
    testsGlob: srcDir + '/**/__tests__/**/*-test.js',
    shouldCompileTests: false,
    babel: {}, // See src/config.
    srcDir: srcDir,
    destDir: destDir,
    sourceMaps: false
  });

  return opts;
}

function task(gulp) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];


  opts = getOpts(opts);

  gulp.task('lib-modules:compile', gulp.series(['lib-modules:compile-js', 'lib-modules:copy-files']));

  gulp.task('lib-modules:compile-js', function () {
    var gulpif = require('gulp-if');
    var changed = require('gulp-changed');
    var ignore = require('gulp-ignore');
    var babel = require('gulp-babel');
    var sourcemaps = require('gulp-sourcemaps');
    var cwd = require('cwd');

    return gulp.src(opts.srcGlob).pipe(gulpif(gulp.isWatching, changed(opts.destDir))).pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob))).pipe(gulpif(opts.sourceMaps, sourcemaps.init())).pipe(babel(opts.babel)).on('error', onError())
    // Source maps work with IntelliJ debugger using Node.js 0.12.7.
    .pipe(gulpif(opts.sourceMaps, sourcemaps.write('.', {
      includeContent: false,
      sourceRoot: function sourceRoot(file) {
        return path.join(path.relative(file.path, cwd()), opts.srcDir);
      }
    }))).pipe(gulp.dest(opts.destDir)).pipe(logFileHelpers());
  });

  gulp.task('lib-modules:copy-files', function () {
    return gulp.src(opts.srcFilesGlob).pipe(gulp.dest(opts.destDir));
  });

  gulp.task('lib-modules:clean', function () {
    var del = require('del');
    var gutil = require('gulp-util');

    gutil.log('Cleaning:', gutil.colors.yellow(opts.destDir));
    return del(opts.destGlob);
  });

  gulp.task('lib-modules:watch', gulp.series('lib-modules:clean', 'lib-modules:compile', function watcher() {
    var gutil = require('gulp-util');

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true;

    gulp.watch(opts.srcGlob, gulp.series('lib-modules:compile')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.yellow(evt));
    });
  }));
}

// Babel logging.
function logFileHelpers() {
  var through = require('through2');
  var gutil = require('gulp-util');

  return through.obj(function (file, enc, cb) {
    var src = file.history[0];
    var dest = file.history[1];
    // TODO(vjpr): When generating source maps this should print src as generated.
    var relPath = function relPath(f) {
      return path.relative('.', f);
    };
    gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)));
    //gutil.log(file.babel.usedHelpers)
    cb(null, file);
  });
}

function onError() {
  var notify = require('gulp-notify');
  return notify.onError('<%= error.message %>');
}