'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.task = task;

var _clearTerminal = require('../util/clear-terminal');

var _clearTerminal2 = _interopRequireDefault(_clearTerminal);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var mergePrefixedCliArgs = require('../util/merge-prefixed-cli-args');

function getModuleOpts(opts) {
  var cwd = require('cwd');
  var _ = require('lodash');

  // TODO(vjpr): Hacky! I think we refactored this so we don't even need a separate `flow-modules` task.
  var srcDir = cwd('modules');
  var srcDir2 = cwd('src');
  var destDir = 'lib-modules';

  // TODO(vjpr): This is very bad! We should make each task generic. Should export just functions.
  // Or this might be fine.
  // TODO(vjpr): Why merge! Should be defaults?
  opts = _.merge({}, opts, {
    srcGlob: [srcDir + '/**/*.js', srcDir2 + '/**/*.js'],
    srcFilesGlob: srcDir + '/**/!(*.js|*.jsx)',
    destGlob: destDir + '/**/*',
    // TODO(vjpr): Test format has changed...but we don't use it so leave it for now.
    testsGlob: srcDir + '/**/__tests__/**/*-test.js',
    shouldCompileTests: false,
    babel: {}, // See src/config.
    srcDir: srcDir,
    destDir: destDir,
    sourceMaps: false
  });

  return opts;
}

function task(gulp, opts) {

  opts = getModuleOpts(opts);
  opts = mergePrefixedCliArgs(opts, 'flow');

  gulp.task('flow', gulp.series('flow:watch'));

  var didLastRunFail = false;

  gulp.task('flow:check', function (done) {
    var _require = require('child_process');

    var spawnSync = _require.spawnSync;
    var spawn = _require.spawn;

    // TODO(vjpr): Limit output to fit on screen to allow user to work through errors from first to last.

    (0, _clearTerminal2.default)();

    var lastOutputLine = null;

    // This allows transformation.
    var child = spawn('flow', ['status', '--color', 'always'], {
      // We set each stream manually because we want to intercept the stdout,
      // but let the stderr pass through which displays the animated loading indicator.
      stdio: [process.stdin, 'pipe', process.stderr]
    });
    child.stdout.on('data', function (d) {
      d = d.toString();
      d = addIdeLinks(d);
      process.stdout.write(d);
      lastOutputLine = d;
    });
    child.on('close', function (code) {
      if (code !== 2) {
        // TODO(vjpr): Should it be 1 or 0?
        if (didLastRunFail) {
          // Notify that errors were fixed.
          notify('Flow errors fixed!'); // TODO(vjpr): Show a green emoji tick.
        }
        didLastRunFail = false;
        return done();
      }
      if (code === 2) {
        // There was errors.
        if (!didLastRunFail) {
          // Only notify if there are new errors.
          var errors = lastOutputLine && lastOutputLine.split('\n').slice(-2)[0];
          var message = 'Flow has errors. ' + errors || 'You should fix them.';
          notify(message);
        }
        didLastRunFail = true;
        return done();
      }
      done();
    });
  });

  function notify(message) {
    var notifier = require('node-notifier');
    notifier.notify({
      // TODO(vjpr): Maybe print gulp command which was typed in?
      // TODO(vjpr): If the project title is long, 'flow has errors' is cut off.
      title: '' + process.cwd().split('/').slice(-1),
      message: message,
      //icon: path.resolve(__dirname, '../../images/flowtype.png'),
      contentImage: _path2.default.resolve(__dirname, '../../images/flowtype.png')
    });
  }

  // TODO(vjpr): Add a notification when the log file changes.
  gulp.task('flow:watch', gulp.series('flow:check', function watcher() {
    var gutil = require('gulp-util');

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true;

    // Sometimes we might want to use src, sometimes something else.
    gulp.watch(opts.srcGlob, gulp.series('flow:check')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.yellow(evt.path));
    });
  }));
}

////////////////////////////////////////////////////////////////////////////////

function addIdeLinks(text) {
  var _require2 = require('live-caller');

  var makeIDELink = _require2.makeIDELink;

  var _require3 = require('path');

  var join = _require3.join;

  var c = require('chalk');
  var stripcolorcodes = function stripcolorcodes(str) {
    return str.replace(/\033\[[0-9;]*m/g, '');
  };
  var out = text.replace(/^(\S+\/[^/]+):(\d+)/gm, function (match, a, b) {
    //match = stripcolorcodes(match)
    a = stripcolorcodes(a);
    b = stripcolorcodes(b);
    var info = { path: join(process.cwd(), a), line: b, col: undefined };
    var ideLink = makeIDELink(info);
    var str = match + '\n' + c.dim(ideLink);
    return str;
  });
  return out;
}