'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.task = task;
var _ = require('lodash');
var cwd = require('cwd');
var path = require('path');
var mergePrefixedCliArgs = require('../util/merge-prefixed-cli-args');

function getOpts(opts) {
  return _.defaults(opts, {
    srcGlob: '',
    testsGlob: '',
    mocha: {
      // TODO(vjpr): `babel-register` should be used instead.
      r: ['babel-register', 'babelator/defaults/mocha-bootstrap']
    },
    //reporter: 'nyan',
    testsDestGlob: ''
  });
}

function task(gulp, opts) {

  opts = getOpts(opts);
  opts = mergePrefixedCliArgs(opts, 'mocha');

  function handleError(e) {
    console.log(e.toString());
    if (gulp.isWatching) {
      this.emit('end');
      return;
    }
    process.exit(1);
  }

  gulp.task('mocha:src', function () {
    var mocha = require('gulp-spawn-mocha');

    return gulp.src(opts.testsGlob, { read: false }).pipe(mocha(opts.mocha)).on('error', handleError);
  });

  // Faster: Run tests on transpiled files so no need for slow babel/require.
  gulp.task('mocha:lib:run', function () {
    var mocha = require('gulp-spawn-mocha');

    if (!opts.shouldCompileTests) throw 'shouldCompileTests must be enabled for `mocha:lib:run` task';

    var mochaOpts = _.clone(opts.mocha);
    mochaOpts.r = _.without(mochaOpts.r, 'babel/register');
    return gulp.src(opts.testsDestGlob, { read: false }).pipe(mocha(mochaOpts)).on('error', handleError);
  });

  gulp.task('mocha:lib', gulp.series('lib:compile', 'mocha:lib:run'));

  gulp.task('mocha:set-watching', function (done) {
    gulp.isWatching = true;
    done();
  });

  gulp.task('test:watch', gulp.series('mocha:set-watching', 'mocha:lib', function watcher() {
    var gutil = require('gulp-util');

    return gulp.watch(opts.srcGlob, gulp.series('mocha:lib')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.yellow(evt));
    });
  }));
}

//// TODO(vjpr): Not teted.
//gulp.task('karma:start', (done) => {
//  const karma = require('karma')
//  new karma.Server({
//    configFile: cwd('karma.conf.js'),
//    singleRun: true,
//  }, done).start()
//})