import clearTerminal from '../util/clear-terminal'
import path, {join} from 'path'
import _ from 'lodash'
const mergePrefixedCliArgs = require('../util/merge-prefixed-cli-args')

function getModuleOpts(opts) {
  const cwd = require('cwd')
  const _ = require('lodash')

  // TODO(vjpr): Hacky! I think we refactored this so we don't even need a separate `flow-modules` task.
  const srcDir = cwd('modules')
  const srcDir2 = cwd('src')
  const destDir = 'lib-modules'

  // TODO(vjpr): This is very bad! We should make each task generic. Should export just functions.
  // Or this might be fine.
  // TODO(vjpr): Why merge! Should be defaults?
  opts = _.merge({}, opts, {
    srcGlob: [srcDir + '/**/*.js', srcDir2 + '/**/*.js'],
    srcFilesGlob: srcDir + '/**/!(*.js|*.jsx)',
    destGlob: destDir + '/**/*',
    // TODO(vjpr): Test format has changed...but we don't use it so leave it for now.
    testsGlob: srcDir + '/**/__tests__/**/*-test.js',
    shouldCompileTests: false,
    babel: {}, // See src/config.
    srcDir,
    destDir,
    sourceMaps: false,
  })

  return opts

}

export function task(gulp, opts) {

  opts = getModuleOpts(opts)
  opts = mergePrefixedCliArgs(opts, 'flow')

  gulp.task('flow', gulp.series('flow:watch'))

  let didLastRunFail = false

  gulp.task('flow:check', (done) => {
    const {spawnSync, spawn} = require('child_process')

    // TODO(vjpr): Limit output to fit on screen to allow user to work through errors from first to last.
    clearTerminal()

    let lastOutputLine = null

    // This allows transformation.
    const child = spawn('flow', ['status', '--color', 'always'], {
      // We set each stream manually because we want to intercept the stdout,
      // but let the stderr pass through which displays the animated loading indicator.
      stdio: [
        process.stdin,
        'pipe',
        process.stderr,
      ]
    })
    child.stdout.on('data', d => {
      d = d.toString()
      d = addIdeLinks(d)
      process.stdout.write(d)
      lastOutputLine = d
    })
    child.on('close', code => {
      if (code !== 2) { // TODO(vjpr): Should it be 1 or 0?
        if (didLastRunFail) {
          // Notify that errors were fixed.
          notify('Flow errors fixed!') // TODO(vjpr): Show a green emoji tick.
        }
        didLastRunFail = false
        return done()
      }
      if (code === 2) {
        // There was errors.
        if (!didLastRunFail) {
          // Only notify if there are new errors.
          const errors = lastOutputLine && lastOutputLine.split('\n').slice(-2)[0]
          const message = 'Flow has errors. ' + errors || 'You should fix them.'
          notify(message)
        }
        didLastRunFail = true
        return done()
      }
      done()
    })

  })

  function notify(message) {
    const notifier = require('node-notifier')
    notifier.notify({
      // TODO(vjpr): Maybe print gulp command which was typed in?
      // TODO(vjpr): If the project title is long, 'flow has errors' is cut off.
      title: `${process.cwd().split('/').slice(-1)}`,
      message,
      //icon: path.resolve(__dirname, '../../images/flowtype.png'),
      contentImage: path.resolve(__dirname, '../../images/flowtype.png'),
    })
  }

  // TODO(vjpr): Add a notification when the log file changes.
  gulp.task('flow:watch', gulp.series('flow:check', function watcher() {
    const gutil = require('gulp-util')

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true

    // Sometimes we might want to use src, sometimes something else.
    gulp.watch(opts.srcGlob, gulp.series('flow:check')).on('change', (evt) => {
      gutil.log('---')
      gutil.log('File changed:', gutil.colors.yellow(evt.path))
    })

  }))

}

////////////////////////////////////////////////////////////////////////////////

function addIdeLinks(text) {
  const {makeIDELink} = require('live-caller')
  const {join} = require('path')
  const c = require('chalk')
  const stripcolorcodes = str => str.replace(/\033\[[0-9;]*m/g, '')
  const out = text.replace(/^(\S+\/[^/]+):(\d+)/gm, (match, a, b) => {
    //match = stripcolorcodes(match)
    a = stripcolorcodes(a)
    b = stripcolorcodes(b)
    const info = {path: join(process.cwd(), a), line: b, col: undefined}
    const ideLink = makeIDELink(info)
    const str = `${match}\n${c.dim(ideLink)}`
    return str
  })
  return out
}
